<?php

namespace Drupal\self_evaluation\Controller;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\Template\Attribute;
use Drupal\self_evaluation\Services\SelfEvaluationDashboardManager;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * User Dashboard Controller.
 */
class SelfEvaluationDashboardController extends ControllerBase {

  /**
   * SelfEvaluation Entity Retriever service.
   *
   * @var \Drupal\self_evaluation\Services\SelfEvaluationDashboardManager
   */
  protected $selfEvaluationDashboardManager;

  /**
   * Constructs a new SelfEvaluationDashboardController.
   *
   * @param \Drupal\Core\Session\AccountProxyInterface $currentUser
   *   User manager service.
   * @param \Drupal\self_evaluation\Services\SelfEvaluationDashboardManager $selfEvaluationDashboardManager
   *   Self Evaluation Dashboard Manager.
   */
  public function __construct(AccountProxyInterface $currentUser, SelfEvaluationDashboardManager $selfEvaluationDashboardManager) {
    $this->currentUser = $currentUser;
    $this->selfEvaluationDashboardManager = $selfEvaluationDashboardManager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): self {
    return new static(
      $container->get('current_user'),
      $container->get('self_evaluation.dashboard_manager')
    );
  }

  /**
   * Render the dashboard for the current user.
   *
   * @return array
   *   A renderable array.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function build(): array {
    $global_informations = [];

    $build = [
      '#type' => 'container',
      '#attributes' => ['class' => ['dashboard-default']],
    ];

    $self_evaluations_list_details = $this->selfEvaluationDashboardManager->getAnswersBySelfEvaluation();

    foreach ($self_evaluations_list_details as $self_evaluation_details) {
      $informations = $this->selfEvaluationDashboardManager->getSelfEvaluationInformations($self_evaluation_details);
      $global_informations[] = $informations['template'];
      $build[] = $informations['default_build'];
    }

    return [
      '#theme' => 'self_evaluation_dashboard',
      '#attributes' => new Attribute(['class' => ['self-evaluation-dashboard']]),
      '#title' => $this->t('Dashboard'),
      '#global_informations' => $global_informations,
      '#current_user' => $this->currentUser,
      '#dashboard_default' => $build,
    ];
  }

  /**
   * Check access to dashboard.
   *
   * @return \Drupal\Core\Access\AccessResultAllowed|\Drupal\Core\Access\AccessResultForbidden
   *   Access result.
   */
  public function access() {
    if ($this->currentUser->isAnonymous()) {
      return AccessResult::forbidden();
    }
    return AccessResult::allowed();
  }

}
