<?php

namespace Drupal\self_evaluation\Plugin\self_evaluation\FormStepsMode;

use Drupal\Core\Form\FormStateInterface;
use Drupal\self_evaluation\SelfEvaluationAnswerInterface;
use Drupal\self_evaluation\SelfEvaluationInterface;

/**
 * Provides a Them Form steps mode.
 *
 * @FormStepsMode(
 *   id = "question",
 *   label = @Translation("Question"),
 *   description = @Translation("Question Form steps Mode")
 * )
 */
class Question extends FormStepsMode {

  /**
   * {@inheritdoc}
   */
  public function getElements(): array {
    $build = parent::getElements();
    if ($this->step == 0) {
      return [
        'form' => $build,
      ];
    }
    $questions = $this->getQuestionsPerStep();
    $form = $this->buildQuestion($questions[$this->step], $this->step);
    return [
      'form' => $form,
      'totalSteps' => $this->countSteps,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(FormStateInterface $form_state) {
    if ($this->step == 0) {
      $this->saveAnswerGroupments($form_state);
    }
    else {
      $questions = $this->getQuestionsPerStep();
      if (!empty($form_state->getValue("choices_{$questions[$this->step]['question']->id()}"))) {
        $this->saveAnswerQuestion($form_state, $questions[$this->step]['question']);
      }
    }
    parent::submitForm($form_state);
  }

  /**
   * {@inheritdoc}
   */
  private function getQuestionsPerStep(): array {
    $number_questions = 1;
    $questions = [];
    $params_themes = [
      'conditions' => [
        [
          'field' => 'self_evaluation',
          'value' => $this->selfEvaluation->id(),
        ],
      ],
      'sorts' => [
        [
          'field' => 'weight',
        ],
        [
          'field' => 'name',
        ],
      ],
    ];
    $themes = $this->selfEvaluationEntityRetriever->getEntities('self_evaluation_theme', $params_themes);
    foreach ($themes as $theme) {
      $params_questions = [
        'conditions' => [
          [
            'field' => 'self_evaluation_theme',
            'value' => $theme->id(),
          ],
        ],
        'sorts' => [
          [
            'field' => 'weight',
          ],
          [
            'field' => 'id',
          ],
        ],
      ];
      $questions_theme = $this->selfEvaluationEntityRetriever->getEntities('self_evaluation_question', $params_questions);
      foreach ($questions_theme as $question) {
        $params_choices = [
          'conditions' => [
            [
              'field' => 'self_evaluation_question',
              'value' => $question->id(),
            ],
          ],
          'sorts' => [
            [
              'field' => 'weight',
            ],
            [
              'field' => 'id',
            ],
          ],
        ];
        $questions[$number_questions] = [
          'theme' => $theme,
          'question' => $question,
          'choices' => $this->selfEvaluationEntityRetriever->getEntities('self_evaluation_question_choice', $params_choices),
        ];
        $number_questions++;
      }
    }
    $this->countSteps = count($questions);
    $this->countQuestions = count($questions);
    $this->countThemes = count($themes);
    return $questions;
  }

  /**
   * {@inheritdoc}
   */
  public function getSteps(SelfEvaluationInterface $self_evaluation, SelfEvaluationAnswerInterface $answer): array {
    parent::getSteps($self_evaluation, $answer);
    return $this->getQuestionsPerStep();
  }

  /**
   * {@inheritdoc}
   */
  public function getResumptionStep(SelfEvaluationInterface $self_evaluation, SelfEvaluationAnswerInterface $answer): int {
    $step = 1;
    $questions = $this->getQuestionsPerStep();
    $question_to_search = $answer->getSelfEvaluationQuestion();
    foreach ($questions as $key => $question) {
      if ($question_to_search == $question['question']) {
        $step = array_key_last($questions) == $key ? -1 : $key + 1;
        break;
      }
    }
    return $step;
  }

}
