<?php

namespace Drupal\self_evaluation\Plugin\self_evaluation\FormStepsMode;

use Drupal\Core\Form\FormStateInterface;
use Drupal\self_evaluation\SelfEvaluationAnswerInterface;
use Drupal\self_evaluation\SelfEvaluationInterface;

/**
 * Provides a Them Form steps mode.
 *
 * @FormStepsMode(
 *   id = "theme",
 *   label = @Translation("Theme"),
 *   description = @Translation("Theme Form steps Mode")
 * )
 */
class Theme extends FormStepsMode {

  /**
   * {@inheritdoc}
   */
  public function getElements(): array {
    $build = parent::getElements();
    if ($this->step == 0) {
      return [
        'form' => $build,
      ];
    }
    $themes = $this->getQuestionsPerTheme();
    $form = $this->buildThemes($themes);
    return [
      'form' => $form,
      'totalSteps' => $this->countSteps,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(FormStateInterface $form_state) {
    if ($this->step == 0) {
      $this->saveAnswerGroupments($form_state);
    }
    else {
      $themes = $this->getQuestionsPerTheme();
      foreach ($themes[$this->step]['questions'] as $question) {
        if (!empty($form_state->getValue("choices_{$question['question']->id()}"))) {
          $this->saveAnswerQuestion($form_state, $question['question']);
        }
      }
    }

    parent::submitForm($form_state);
  }

  /**
   * Returns questions sorted by themes.
   *
   * @return array
   *   An array of questions sorted by themes
   */
  private function getQuestionsPerTheme(): array {
    $number_themes = 1;
    $number_question = 1;
    $total_questions = 0;
    $themes = [];

    $params_themes = [
      'conditions' => [
        [
          'field' => 'self_evaluation',
          'value' => $this->selfEvaluation->id(),
        ],
      ],
      'sorts' => [
        [
          'field' => 'weight',
        ],
        [
          'field' => 'name',
        ],
      ],
    ];
    $list_themes = $this->selfEvaluationEntityRetriever->getEntities('self_evaluation_theme', $params_themes);
    foreach ($list_themes as $theme) {
      $questions = [];
      $params_questions = [
        'conditions' => [
          [
            'field' => 'self_evaluation_theme',
            'value' => $theme->id(),
          ],
        ],
        'sorts' => [
          [
            'field' => 'weight',
          ],
        ],
      ];
      $questions_theme = $this->selfEvaluationEntityRetriever->getEntities('self_evaluation_question', $params_questions);

      foreach ($questions_theme as $question) {
        $params_choices = [
          'conditions' => [
            [
              'field' => 'self_evaluation_question',
              'value' => $question->id(),
            ],
          ],
          'sorts' => [
            [
              'field' => 'weight',
            ],
            [
              'field' => 'id',
            ],
          ],
        ];
        $questions[$question->id()] = [
          'theme' => $theme,
          'question' => $question,
          'number_question' => $number_question,
          'choices' => $this->selfEvaluationEntityRetriever->getEntities('self_evaluation_question_choice', $params_choices),
        ];
        $number_question++;
        $total_questions++;
      }
      $themes[$number_themes] = [
        'theme' => $theme,
        'questions' => $questions,
      ];
      $number_themes++;
    }
    $this->countSteps = count($themes);
    $this->countQuestions = $total_questions;
    $this->countThemes = count($themes);
    return $themes;
  }

  /**
   * {@inheritdoc}
   */
  public function getSteps(SelfEvaluationInterface $self_evaluation, SelfEvaluationAnswerInterface $answer): array {
    parent::getSteps($self_evaluation, $answer);
    return $this->getQuestionsPerTheme();
  }

  /**
   * {@inheritdoc}
   */
  public function getResumptionStep(SelfEvaluationInterface $self_evaluation, SelfEvaluationAnswerInterface $answer): int {
    $step = 1;
    $themes = $this->getQuestionsPerTheme();
    $question_to_search = $answer->getSelfEvaluationQuestion();
    foreach ($themes as $key => $theme) {
      if (array_key_exists($question_to_search->id(), $theme['questions'])) {
        $step = array_key_last($theme['questions']) != $question_to_search->id()
          ? $key
          : (array_key_last($themes) == $key
            ? -1
            : $key + 1
          );
        break;
      }
    }
    return $step;
  }

}
