<?php

namespace Drupal\self_evaluation;

use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Entity\EntityChangedInterface;
use Drupal\user\EntityOwnerInterface;

/**
 * Provides an interface defining an answer entity type.
 */
interface SelfEvaluationAnswerInterface extends ContentEntityInterface, EntityOwnerInterface, EntityChangedInterface {

  const SHARABLE = 1;

  const UNSHARABLE = 0;

  const PUBLISHABLE = 1;

  const UNPUBLISHABLE = 0;

  /**
   * Gets the self evaluation answer creation timestamp.
   *
   * @return int
   *   Creation timestamp of the self evaluation answer.
   */
  public function getCreatedTime(): int;

  /**
   * Sets the self evaluation answer creation timestamp.
   *
   * @param int $timestamp
   *   The self evaluation answer creation timestamp.
   *
   * @return \Drupal\self_evaluation\SelfEvaluationAnswerInterface
   *   The called self evaluation answer entity.
   */
  public function setCreatedTime(int $timestamp): SelfEvaluationAnswerInterface;

  /**
   * Returns the self evaluation answer status.
   *
   * @return bool
   *   TRUE if the self evaluation answer is enabled, FALSE otherwise.
   */
  public function isEnabled(): bool;

  /**
   * Returns the self evaluation answer share.
   *
   * @return bool
   *   TRUE if the self evaluation answer is shared, FALSE otherwise.
   */
  public function isShared(): bool;

  /**
   * Returns the self evaluation answer publish.
   *
   * @return bool
   *   TRUE if the self evaluation answer is published, FALSE otherwise.
   */
  public function isPublished(): bool;

  /**
   * Sets the self evaluation answer status.
   *
   * @param bool $status
   *   TRUE to enable this self evaluation answer, FALSE to disable.
   *
   * @return \Drupal\self_evaluation\SelfEvaluationAnswerInterface
   *   The called self evaluation answer entity.
   */
  public function setStatus(bool $status): SelfEvaluationAnswerInterface;

  /**
   * Gets the related self evaluation entity.
   *
   * @return \Drupal\self_evaluation\SelfEvaluationInterface|null
   *   The self evaluation entity.
   */
  public function getSelfEvaluation(): ?SelfEvaluationInterface;

  /**
   * Gets the related question entity.
   *
   * @return \Drupal\self_evaluation\SelfEvaluationQuestionInterface|null
   *   The question entity.
   */
  public function getSelfEvaluationQuestion(): ?SelfEvaluationQuestionInterface;

  /**
   * Gets the token.
   *
   * @return string
   *   The token.
   */
  public function getToken(): string;

  /**
   * Gets the scores per thematics.
   *
   * @return array
   *   The self evaluation answer thematic score results.
   */
  public function getThematicScores() : array;

  /**
   * Gets the scores per question.
   *
   * @return array
   *   The self evaluation answer question score results.
   */
  public function getQuestionScores() : array;

  /**
   * Gets answer groupments.
   *
   * @return \Drupal\self_evaluation\SelfEvaluationGroupmentInterface[]
   *   The self evaluation answer groupments.
   */
  public function getGroupments(): array;

  /**
   * Gets the global score.
   *
   * @return array
   *   The self evaluation answer global score.
   */
  public function getGlobalScore() : array;

}
