<?php

namespace Drupal\self_evaluation;

use Drupal\Core\Entity\ContentEntityInterface;

/**
 * Provides an interface defining a question entity type.
 */
interface SelfEvaluationQuestionInterface extends ContentEntityInterface, SelfEvaluationThemableInterface {

  /**
   * Gets the self evaluation question title.
   *
   * @return string
   *   Title of the self evaluation question.
   */
  public function getTitle();

  /**
   * Gets the self evaluation question short title.
   *
   * @return string
   *   Title of the self evaluation question.
   */
  public function getShortTitle();

  /**
   * Sets the self evaluation question title.
   *
   * @param string $title
   *   The self evaluation question title.
   *
   * @return \Drupal\self_evaluation\SelfEvaluationQuestionInterface
   *   The called self evaluation question entity.
   */
  public function setTitle(string $title): SelfEvaluationQuestionInterface;

  /**
   * Returns the self evaluation question status.
   *
   * @return bool
   *   TRUE if the self evaluation question is enabled, FALSE otherwise.
   */
  public function isEnabled();

  /**
   * Sets the self evaluation question status.
   *
   * @param bool $status
   *   TRUE to enable this self evaluation question, FALSE to disable.
   *
   * @return \Drupal\self_evaluation\SelfEvaluationQuestionInterface
   *   The called self evaluation question entity.
   */
  public function setStatus(bool $status): SelfEvaluationQuestionInterface;

}
