<?php

namespace Drupal\self_evaluation;

use Drupal\Core\Entity\ContentEntityInterface;

/**
 * Provides an interface defining a theme entity type.
 */
interface SelfEvaluationThemeInterface extends ContentEntityInterface {

  /**
   * Gets the theme name.
   *
   * @return string
   *   Name of the theme.
   */
  public function getName(): string;

  /**
   * Gets the theme short name.
   *
   * @return string
   *   Short name of the theme.
   */
  public function getShortName(): string;

  /**
   * Sets the theme title.
   *
   * @param string $name
   *   The theme title.
   *
   * @return \Drupal\self_evaluation\SelfEvaluationThemeInterface
   *   The called theme entity.
   */
  public function setName(string $name): SelfEvaluationThemeInterface;

  /**
   * Returns the theme status.
   *
   * @return bool
   *   TRUE if the theme is enabled, FALSE otherwise.
   */
  public function isEnabled();

  /**
   * Sets the theme status.
   *
   * @param bool $status
   *   TRUE to enable this theme, FALSE to disable.
   *
   * @return \Drupal\self_evaluation\SelfEvaluationThemeInterface
   *   The called theme entity.
   */
  public function setStatus(bool $status): SelfEvaluationThemeInterface;

  /**
   * Returns the renderable array of the description.
   *
   * @return array
   *   The renderable array of the description.
   */
  public function getDescriptionRenderArray(): array;

  /**
   * Gets the related self evaluation entity.
   *
   * @return \Drupal\self_evaluation\SelfEvaluationInterface|null
   *   The self evaluation entity.
   */
  public function getSelfEvaluation(): ?SelfEvaluationInterface;

}
